/*
   FNT - SQL-backend for MySQL - function query

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <mysql.h>
#include <errmsg.h>		/* mysql/errormsg.h */
#include <stdlib.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_sql_query_t *
query (fnt_sql_connect_t * sql_connect, const char *query_string)
{

    fnt_sql_query_t *ret_value;
    MYSQL_RES *result;

    if ((sql_connect == NULL) || (query_string == NULL))
	return NULL;

    ret_value = g_try_malloc (sizeof (fnt_sql_query_t));
    if (ret_value == NULL)
	return NULL;

    if (mysql_query ((MYSQL *) sql_connect->db_handle, query_string))
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			"%s", mysql_error ((MYSQL *) sql_connect->db_handle));
	g_free (ret_value);
	return NULL;
    }

    result = mysql_store_result (sql_connect->db_handle);
    if (result)
    {
	/* there are rows */
	ret_value->is_result_present = true;
	ret_value->num_fields = mysql_num_fields (result);
	ret_value->num_rows = mysql_num_rows (result);
    }
    else
    {

	if (mysql_field_count (sql_connect->db_handle) == 0)
	{
	    /* it was not a SELECT */
	    ret_value->is_result_present = false;
	    ret_value->num_fields = 0;
	    ret_value->num_rows =
		mysql_affected_rows (sql_connect->db_handle);
	}
	else
	{
	    g_free (ret_value);
	    return NULL;
	}
    }
    ret_value->handle = (void *) result;
    return ret_value;
}
